package pacoteCliente;

import java.io.BufferedReader;

import pacoteConexaoControle.ThreadConexaoControleEntrada;

public class RepositorioConexoes {

	Cliente cliente;
	Conexoes[] repositorio;
	int tamanho;
	int proxPosVaga;
	
	public RepositorioConexoes(Cliente cliente){
		this.cliente = cliente;
		this.tamanho = 6;
		this.proxPosVaga = 0;
		this.repositorio = new Conexoes[tamanho];
	}
	
	//fecha apenas uma unica conexao
	public void fecharConexao(String nome){
		int pos = this.procurarConexao(nome);
		this.repositorio[pos].encerrarConexao();
	}
	
	//fecha todas as conexoes!
	public void finalizar(){
		for(int i = 0; i < proxPosVaga; i++){
			this.repositorio[i].encerrarConexao();
		}
	}
	
	public void adicionarNovaConexao(String nome,String ip,int portaChat,int portaDados, int portaControle, Cliente cliente){
		this.repositorio[proxPosVaga] = new Conexoes(nome, ip,portaChat,portaDados, portaControle, cliente);
		proxPosVaga++;
	}
	
	public void atualizar(Usuario[] repositorioUsuarios){
		Conexoes[] temp = new Conexoes[tamanho]; //como ja sabe q n havera mais q 'tamanho' conexoes, nao dara problema..
		int proxPos = 0;
		for(int i = 0; i < repositorioUsuarios.length; i++){
			String nome = repositorioUsuarios[i].getUser();
			String host = repositorioUsuarios[i].getHost();
			int portaChat = repositorioUsuarios[i].getPortaChat();
			int portaDados = repositorioUsuarios[i].getPortaDados();
			int portaControle = repositorioUsuarios[i].getPortaControle();
			
			if(this.existeConexao(nome)){
				int posicao = this.procurarConexao(nome);
				temp[proxPos] = this.repositorio[posicao];
			} else { //significa q nao ha conexao aberta ainda!
				temp[proxPos] = new Conexoes(nome, host,portaChat,portaDados,portaControle, cliente);
			}
			proxPos++;
		}
		this.repositorio = temp;
		this.proxPosVaga = proxPos;
	}
	
	private int procurarConexao(String nome){
		boolean achou = false;
		int posicao = 0;
		while(!achou && posicao < this.proxPosVaga){
			if(this.repositorio[posicao].getNome().equals(nome)){
				achou = true;
			}
			posicao++;
		}
		return posicao -1;
	}
	
	public void setConexaoEntrada(String nome,Cliente cliente,BufferedReader entrada){
		try {
			synchronized(this){
				wait(500);
			}
		} catch (InterruptedException e) {
			System.out.println("erro ao tentar esperar p setar conexaoEntrada");
			e.printStackTrace();
		}
		if(this.existeConexao(nome)){
			int posicao = this.procurarConexao(nome);
			this.repositorio[posicao].setEntrada(new ThreadConexaoEntrada(nome,cliente,entrada));
		}// else {
//			System.out.println("fazer as conexoes separadas msm ;/ \n");
//		}
	}
	
	public boolean existeConexao(String nome){
		boolean existe = false;
		int posicao = 0;
		while(!existe && posicao < this.proxPosVaga){
			if(this.repositorio[posicao].getNome().equals(nome)){
				existe = true;
			}
			posicao++;
		}
		return existe;
	}
	
	public void escrever(String frase){
		for(int i = 0; i < proxPosVaga; i++){
			this.repositorio[i].escrever(frase);
		}
	}

	public void setConexaoControleEntrada(String ip, ThreadConexaoControleEntrada novaEntrada) {
		try {
			synchronized(this){
				wait(500);
			}
		} catch (InterruptedException e) {
			System.out.println("erro ao tentar esperar p setar conexaoControleEntrada");
			e.printStackTrace();
		}
		if(this.existe(ip)){
			int posicao = this.procurarConexao(ip);
			this.repositorio[posicao].setControleEntrada(novaEntrada);
		} else {
			System.out.println("erro ao tentar setar a cpmexapControleEntrada");
		}
		
	}

	public void mandarAck(String ip, boolean flag, int id, boolean ping,
			int sequencia) {
		System.out.println(this.existe(ip));
		if(this.existe(ip)){
			int posicao = this.procurar(ip);
			this.repositorio[posicao].mandarAck(flag,id,ping,sequencia);
		}
		
	}

	private int procurar(String ip) {
		boolean achou = false;
		int posicao = 0;
		while(!achou && posicao < this.proxPosVaga){
			if(this.repositorio[posicao].getIp().equals(ip)){
				achou = true;
			}
			posicao++;
		}
		//if(posicao == this.proxPosVaga) posicao = 0;
		return (posicao-1);
	}

	private boolean existe(String ip) {
		boolean achou = false;
		int posicao = 0;
		while(!achou && posicao < this.proxPosVaga){
			if(this.repositorio[posicao].getIp().equals(ip)){
				achou = true;
			}
			posicao++;
		}
		return achou;
	}
	
}
